static char rcs_id[] = "$Header: /home/kig/work/convert/src/RCS/convert.c,v 1.1 1996/10/20 19:37:31 kig Exp kig $";
/*******************************************************************************
 *
 * FILENAME:	convert.c
 * MODULE:		convert
 * NOTES:		file with routines to actually convert text in -c mode
 *				using codes data structure
 *
 * Copyright (c) 1996 Konstantin Gredeskoul
 * All rights reserved
 *
 * Internet:	kig@cyber.com.au
 * Post:		P.O.Box 500, Carnegie, VIC 3163, Australia
 *
 ******************************************************************************/


/* includes */
#include <stdio.h>
#include <sys/stat.h>

#include "debuglog.h"
#include "bool.h"
#include "main.h"


void convert(
	FILE *fp_infile, 			/* input file pointer */
	FILE *fp_outfile, 			/* output file pointer */
	CODE_DATA *codes, 			/* pointer to codes structure */
	int reverse_mode,			/* boolean var, use reverse if this is TRUE */
	char sepchar				/* exclude character */
	)
{
/*******************************************************************************
 *
 * PURPOSE
 *
 ******************************************************************************/

	int			cin = 0;
	int			cout = 0;
	char		ctmp = 0;

	int			bool_exclude = FALSE;
	int			bool_convert = TRUE;

	/*
	 * these register things won't really affect anything,
	 * since most of the compilers will optimize it anyway...
	 * I kind of like it, so let them be.
	 */
	long		cnt = 0;
	int			i = 0;
	long		infile_size = -1;
	char		*pleft = NULL;
	char		*pright = NULL;

	char		func[] = "convert";
	char		*conv_status = "ON ";

	int			perc_curr = -1;
	int			perc_prev = -1;

	int			fd;
	struct stat	stat_buff;


	debuglog((L_DTL, func, "entering"));

	/* get file size to use in status display */
	fd = fileno(fp_infile);

	if (fstat(fd, &stat_buff) != 0)
		debuglog((L_WRN, func, "couldn't get file stat for input file"));
	else
	{
		infile_size = stat_buff.st_size;
		debuglog((L_INF, func, "got input file size = %dKb",
			infile_size / KILOBYTE ));
	}

	/* see if sepchar is given */
	if (sepchar != '\0')
		bool_exclude = TRUE;

	/* if reverse mode is specified, interchange columns
	 * of codes data structure */
	if (reverse_mode == TRUE)
	{
		debuglog((
			L_TRC,
			func,
			"using reverse mode, interchanging columns..."
			));

		for (i = 0; i < codes->len; i++)
		{
			ctmp 			= codes->left[i];
			codes->left[i]	= codes->right[i];
			codes->right[i]	= ctmp;
		}
	}

	/*
	 * Call check_codes() function to check codes for validity.
	 * Note that this function never returns a failure, it only
	 * complains that some problems are found; convert effectively
	 * ignores them. What this means is that we don't really care
	 * about the return code of this function hence it is void.
	 */
	(void)check_codes(codes);

	/* set working pointers to the start of 
	 * codes->left and codes->rihght arrays. */

	pleft = (char *)&codes->left[0];
	pright = (char *)&codes->right[0];
 
	debuglog((L_INF, func, "starting conversion"));

	/* start the loop */
	while ((cin = getc(fp_infile)) != EOF)
	{
		cnt++;

		/* handle status display update */
		if (cnt % PERCENT_BLOCK == 0)
		{
			perc_curr = (int)(100.0 * (float)cnt/(float)infile_size);
			if (perc_curr > perc_prev)
			{
				fprintf(stderr, "status: %d%% complete...\r", perc_curr);
				perc_prev = perc_curr;
			}
		}

		/* set by default out character to
		 * in character. This value may or may not be changed below.
		 */
		cout = cin;

		/* handle exclude character settings */
		if (bool_exclude)
		{
			if (cin == sepchar)
			{
				bool_convert = !bool_convert;

				if (bool_convert)
					conv_status = "ON ";
				else
					conv_status = "OFF";

				debuglog((
					L_TRC,
					func,
					"exclude chr %c at byte %ld - converting is %s",
					sepchar,
					cnt,
					conv_status
					));
			}
		}


		/* run through the codes and see if there is
		 * a match between codes->left and cin.  */

		if (bool_convert)
			for (i = 0; i < codes->len; i++)
				if (*(pleft + i) == (char)cin)
				{
					cout = *(pright + i);
					break;
				}

		/* put out character to the outfile */
		if (!bool_exclude || cin != sepchar)
			putc(cout, fp_outfile);
	}

	debuglog((L_INF, func, "conversion finished"));

	/* update display with 100 (just to make sure) */
	fprintf(stderr, "status: %d%% complete...\n", 100);

	debuglog((L_DTL, func, "leaving"));

	return;
}


void check_codes(CODE_DATA *codes)
/*******************************************************************************
 *
 * DESCRIPTION
 *	Check codes for integrity: this routine reports if multiple matches 
 *	of the same character are found in the left part of codes. 
 *
 *	Eg, if "a" becomes "b" and later in 
 *	the code file "a" becomes also "c" than this will be reported as a 
 *	warning. This does not mean that the code file is wrong, just a warning
 *	to acknowledge that fact that only the first "equation" will be actually
 *	used in the conversion.
 *
 ******************************************************************************/
{
	int			i, j;
	char		func[] = "check_codes";
	int			problems = 0;

	for (i = 0; i < codes->len; i++)
	{
		if (*(codes->left + i) == *(codes->right + i))
		{
			problems++;
			debuglog((L_WRN, func,
				"relation %d assignes <%c> to itself",
				i, *(codes->left + i)));
		}

		for (j = i + 1; j < codes->len; j++)
			if (*(codes->left + j) == *(codes->left + i))
			{
				problems++;

				debuglog((L_WRN, func,
					"repeated relations found: no. %d and %d", i+1, j+1));

				fprintf(stderr, 	
					"warning: repeated relations found: relations %d and %d\n",
					i, j);
			}
	}
	if (problems == 0)
		debuglog((L_INF, func, "NO anomalous data found in code file"));
	else
		debuglog((L_WRN, func, "code file contains potential problems"));

	return;
}

