static char rcs_id[] = "$Header: /home/kig/work/convert/src/RCS/debuglog.c,v 1.1 1996/10/20 19:37:31 kig Exp kig $";
/*******************************************************************************
 *
 * FILENAME:	debuglog.c
 * MODULE:		convert
 *
 ******************************************************************************/


/* Debug logging routines */

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>
#include <stdarg.h>
/* 
#include <fcntl.h>
#include <unistd.h>
*/

#include "debuglog.h"

/* static globals */

static FILE	*fp = NULL;
static int	debug_output_threshold = 0;


/* routines */

int
debuglog_open
(
char	*filename
)
{

	if (filename == NULL)
	{
		fp = stdout;
	}
	else
	{
		fp = fopen(filename, "a");
		if (fp == NULL)
		{
			fprintf(stderr, "can't open file <%s> for logging\n", filename);
			return(-1);
		}
	}

	return(0);
}

int
debuglog_close
(
void
)
{
	/* Close the debug log. */

	if (fp == NULL)
		return(0);


	if (fp != stdout)
	{
		if (fclose(fp) < 0)
		{
			return(-1);
		}
	}

	return(0);
}

void
debuglog_level_set
(
int threshold
)
{
    /* Set the threshold level at which output is written by debuglog.
     * Only messages of severity worse than or equal to the specified
     * threshold will be output.
     *
     * Note:
     * 
     * One way to invoke this function would be with a parameter
     * of type debug_level (the enum that specifies the output message
     * serverity). This is convenient and logical, but since we want the
     * main usage of this function to be to set the output threshold based
     * on a number specified on the command line invocation of a program,
     * it is simpler to just accept an int and use that. Since the enum
     * is an int anyway, this all works out in the end.
     * To make this clearer, here is an example.

     * Let the program 'prog' use the flag -d for debug level. Then if the
     * program is invoked by
     *
     *	prog ... -d 3
     *
     * and the user of this routine calls debug_level_set(3), then
     * only messages of numeric severity 3 or less will be output. A glance at
     * the definition of enum debug_level will reveal that this means that
     * only messages of severity LOG_ERR or _worse_ will be output.
     * The reason we use numbers and not symbols is to save the inevitable
     * translation step from a command line syntax like '-d LOG_ERROR'. This
     * may not seem like a big deal but it simplifies the interface, and
     * make it easier for users: giving a higher number gives more output,
     * and they don't have to remember codes like LOG_ERR.
     */

    debug_output_threshold = threshold;

    return;
}

void
_debuglog
(
debug_level	severity,
const char *func,
const char *format,
...
)
{
	/* Write debug information to a log file, using a printf style
	 * varargs interface. Each line is prefaced with the date and time,
	 * and the log message severity. The user should supply the function name,
	 * the message to be logged, and the severity level.
	 *
	 * Note that only messages of severity worse (numerically less than)
	 * than the threshold set with the function debuglog_level_set will
	 * be output.
	 */

	va_list		args;	/* varargs argument list */
	time_t		t;
	char		*s = "INF";
	char		date[18];

	/* check threshold first */

	if (severity > debug_output_threshold)
	{
	    return;
	}

	/* check if file was opened */
	if (fp == NULL)
	{
		return;
	}

	/* write date */
	t = time(NULL);
	strftime(date, sizeof(date), "%d/%m/%y %H:%M:%S", localtime(&t));


	/* write severity */

	switch(severity)
	{
		case L_DTL:	/* detail */
			s = "DTL";
			break;

		case L_TRC:	/* trace */
			s = "TRC";
			break;

		case L_INF:	/* info */
			s = "INF";
			break;

		case L_WRN:	/* warning */
			s = "WRN";
			break;

		case L_ERR:	/* error */
			s = "ERR";
			break;

		case L_SVR:	/* severe */
			s = "SVR";
			break;

		case L_FTL:	/* fatal */
			s = "FTL";
			break;

		default:	/* should not happen */
			s = "???";
			break;
	}

	fprintf(fp, "%s|%s|%s: ", date, s, func);


	/* write message */

	va_start(args, format);

	vfprintf(fp, format, args);

	va_end(args);


	/* add newline */

	fputs("\n", fp);


	/* flush stream */

	fflush(fp);

	/* done */

	return;
}
